import React, {
  Component,
  PropTypes,
} from 'react';
import {
  View,
  Text,
} from 'react-native';
import lodash from 'lodash';

/**
 * @desc border正则, 就是为了获取字符串中 width 和 color
 */
const BORDER_REGEXP = [
  { key: 'width', regexp: /\d+/ },
  { key: 'color', regexp: /#\w+/ },
];

/**
 * @param {string} borderStr 形如 '2 #ccc'
 * @return { width, color} 只有一个宽高
 * @desc 根据字符串获取宽高
 */
const getBorderStyle = (borderStr) => {
  const style = {};
  let bds = borderStr;
  for (let i = 0; i < BORDER_REGEXP.length; i += 1) {
    const prop = BORDER_REGEXP[i];
    if (prop.regexp.test(bds)) {
      bds = bds.replace(prop.regexp, (match) => {
        style[prop.key] = prop.key === 'width' ? +match : match;
      });
    }
  }
  return style;
};

/**
 * @param {number} flex 占位符flex
 * @param {width, color} borderStyle 占位格子宽度和颜色
 * @param {style} gridsStyle 样式, 继承自最后一排第一个
 * @desc 画占位格子
 */
const gridHolder = (flex, borderStyle, gridsStyle) => {
  // gird 设置padding 的话, 这里需要比较复杂的操作,
  // 因为懒,所以我们给用户的选择是: 不要写 padding margin border 相关属性
  const paddingRight = ((flex - 1) * (borderStyle.width));
  return (<View
    key={new Date()}
    style={[{ paddingRight,
      borderLeftColor: borderStyle.color,
      borderLeftWidth: borderStyle.width }, gridsStyle]}
    flex={flex}
  />);
};

/**
 * @param {Grid} children 内部Grid
 * @param {string} border 就那个string
 * @param {style} gridStyle 格子的样式
 * @desc 画格子们
 */
const renderGrids = (children, border, gridStyle) => (Array.isArray(children)
  /* eslint-disable react/prop-types */
  ? children.map(({ props }, index) => {
    const idxKey = index;
    return <Grid key={idxKey} {...props} border={border} style={[gridStyle, props.style]} />;
  })
  /* eslint-enable */
  : <Grid {...children.props} border={border} style={[gridStyle, children.props.style]} />
);

/**
 * @param {Grid} children 
 * @param {num} col 每列显示的个数
 * @param {string} border 就那个border样式,只有width 和 color
 * @param {number} gridStyle 格子的样式
 * @desc 画一行行格子里面的行
 */
const renderRows = (children, col, border, gridStyle) => {
  if (Array.isArray(children)) {
    const rows = lodash.chunk(children, col);
    const borderStyle = getBorderStyle(border);

    return rows.map((grids, index) => {
      const isLast = index === rows.length - 1;
      if (isLast) {
        const blankFlex = col - grids.length;
        const blankStyle = { ...gridStyle, ...grids[grids.length - 1].props.style };
        if (blankFlex > 0) grids.push(gridHolder(blankFlex, borderStyle, blankStyle));
      }
      const idxKey = index;
      return (<View
        name="row"
        key={idxKey}
        style={{
          flexDirection: 'row',
          borderTopWidth: borderStyle.width,
          borderTopColor: borderStyle.color,
        }}
      >
        {renderGrids(grids, border, gridStyle)}
      </View>);
    });
  }
  if (typeof children === 'string') {
    return <Text>{children}</Text>;
  }
  return children;
};

class Grid extends Component {
  static propTypes = {
    children: PropTypes.oneOfType([PropTypes.element, PropTypes.array, PropTypes.string]),
    style: View.propTypes.style,
    col: PropTypes.number,
    border: PropTypes.oneOfType([PropTypes.string, PropTypes.bool]),
    grid: View.propTypes.style,
  }
  static defaultProps = {
    children: null,
    style: null,
    col: 4,
    border: null,
    grid: null,
  }
  render() {
    const isRoot = !!(Array.isArray(this.props.children)
      && this.props.children.find(({ type }) => type === Grid));
    const {
      children,
      col,
      border,
      grid,
      ...others
    } = this.props;
    const borderStr = border === true ? '1 #cecece' : border;
    const borderStyle = getBorderStyle(borderStr);
    return (
      <View
        {...others}
        style={[isRoot
          ? {
            borderBottomWidth: borderStyle.width,
            borderBottomColor: borderStyle.color,
            borderRightWidth: borderStyle.width,
            borderRightColor: borderStyle.color,
          }
          : {
            flex: 1,
            borderLeftWidth: borderStyle.width,
            borderLeftColor: borderStyle.color,
            justifyContent: 'center',
            alignItems: 'center',
          },
        this.props.style]}
      >
        {renderRows(children, col, borderStr, grid)}
      </View>
    );
  }
}

export default Grid;
